% filtrarea trece-jos a unui semnal unidimensional, 2 variante
% varianta 1: se foloseste filtru cauzal, ordinul 1
% filtrare in domeniul timp, prin convolutie
% varianta 2: se foloseste filtrarea in domeniul frecventa
% se aplica FFT, filtrare ideala trece-jos, FFT invers
% semnalul original este armonic, frecventa 10Hz
% perturbatia este armonica, frecventa 90Hz

% Fs este frecventa de esantionare
% N este numarul de esantioane ale semnalului
% x este sirul esantioanelor semnalului
% y este sirul esantioanelor semnalului filtrat

clear all
close all

N=10000;                                % numarul de esantioane
Ts=1/1000;                              % perioada de esantionare
timp=(1:N)*Ts;                          % sirul valorilor timpului, s
fr=10;                               % frecventa semnalului neperturbat, Hz
x=sin(2*pi*fr*timp);                 % semnalul original
figure
plot(timp(N-200:N),x(N-200:N))          % afiseaza semnalul, ultimele 200ms
title('semnal neperturbat')
xlabel('timpul')
pause
x2=0.2*sin(2*pi*9*fr*timp);          % componenta perturbatoare, de frecventa mare
x=x+x2;                                 % semnal perturbat, suma celor doua componente
figure
plot(timp(N-200:N),x(N-200:N))          % afiseaza semnalul perturbat, ultimele 200ms
title('semnal perturbat')
xlabel('timpul')
pause

% =========================================================================
% filtru cauzal, convolutie in timp

y(1)=x(1);                              % primul esantion
for i=2:N                               % filtrarea trece jos
    y(i)=0.9*y(i-1)+0.1*x(i);           % ecuatia in diferente, filtru de ord. 1
end
figure
plot(timp(N-200:N),y(N-200:N))          % afiseaza semnalul filtrat, ultimele 200ms
title('semnal filtrat prin convolutie 1, domeniul timp')
xlabel('timpul')
pause

X=abs(fft(x));                          % modulul FFT pentru semnalul perturbat
Y=abs(fft(y));                          % modulul FFT pentru semnalul filtrat
frecv=(1:N)/Ts/N;                          % sirul valorilor frecventei transformatei
figure
plot(frecv,X)
title('modulul transformatei Fourier, semnal de intrare')
xlabel('frecventa, Hz')
pause
plot(frecv(1:1500),X(1:1500))
title('modulul transformatei Fourier, semnal de intrare')
xlabel('frecventa, Hz')
figure
plot(frecv(1:1500),Y(1:1500))
title('modulul transformatei Fourier, semnal filtrat 1')
xlabel('frecventa, Hz')
pause

% =========================================================================
% filtru trece-jos, in domeniul frecventa
X1=fft(x);                              % FFT pentru semnalul perturbat
filtru=zeros(1,N);                      % dincolo de frecventa 5*fr = 50Hz
                                        % functia de transfer a filtrului are valoarea 0
for i=1:5*fr*N*Ts                       % pina la frecventa 5*fr = 50Hz
    filtru(i)=1;                        % functia de transfer a filtrului are valoarea 1
end
for i=1:5*fr*N*Ts
    filtru(N-i+1)=1;                    % atit pentru frecvente pozitive, cit si negative!
end

plot(frecv,filtru)                      % afiseaza functia de transfer a filtrului
title('caracteristica amplificare-frecventa a filtrului 2')
xlabel('frecventa, Hz')
axis([0 1000 0 1.2])
pause
plot(frecv(1:1500),filtru(1:1500))      % afiseaza functia de transfer a filtrului, numai pina la 150Hz
title('caracteristica amplificare-frecventa a filtrului 2')
xlabel('frecventa, Hz')
axis([0 150 0 1.2])
pause

Y1=X1.*filtru;                          % FFT pentru semnalul filtrat
%size(Y1)
figure
plot(frecv(1:1500),abs(Y1(1:1500)))     % afiseaza modulul FFT semnal filtrat, numai pina la 150Hz
                                        % se observa diminuarea liniei spectrale a perturbatiei
title('functia spectrala a semnalului filtrat 2')
xlabel('frecventa, Hz')
axis([0 150 0 6000])
pause

y=ifft(Y1);                             % transformata FFT inversa
y2=real(y);
figure
plot(timp(N-200:N),y2(N-200:N))          % afiseaza semnalul filtrat, ultimele 200ms
title('semnal filtrat in domeniul frecventa')
xlabel('timpul')
pause

disp('gata')
disp(' ')
